import * as ts from "typescript";
declare type KnownKeys<T> = {
    [K in keyof T]: string extends K ? never : number extends K ? never : K;
} extends {
    [K in keyof T]: infer U;
} ? U : never;
declare type OmitIndexSignature<T extends Record<any, any>> = Pick<T, KnownKeys<T>>;
export interface TransformerImport {
    transform: string;
    import?: string;
    after?: boolean;
    afterDeclarations?: boolean;
    type?: "program" | "config" | "checker" | "raw" | "compilerOptions";
    [option: string]: any;
}
export interface LuaPluginImport {
    name: string;
    import?: string;
    [option: string]: any;
}
export declare type CompilerOptions = OmitIndexSignature<ts.CompilerOptions> & {
    noImplicitSelf?: boolean;
    noHeader?: boolean;
    luaBundle?: string;
    luaBundleEntry?: string;
    luaTarget?: LuaTarget;
    luaLibImport?: LuaLibImportKind;
    sourceMapTraceback?: boolean;
    luaPlugins?: LuaPluginImport[];
    plugins?: Array<ts.PluginImport | TransformerImport>;
    [option: string]: any;
};
export declare enum LuaLibImportKind {
    None = "none",
    Always = "always",
    Inline = "inline",
    Require = "require"
}
export declare enum LuaTarget {
    Universal = "universal",
    Lua51 = "5.1",
    Lua52 = "5.2",
    Lua53 = "5.3",
    Lua54 = "5.4",
    LuaJIT = "JIT"
}
export declare const isBundleEnabled: (options: CompilerOptions) => boolean;
export declare function validateOptions(options: CompilerOptions): ts.Diagnostic[];
export {};
